/***********************************************************************
 *      This example program is an excerpt from the book:              *
 *                                                                     *
 *          X Window OSF/Motif Toolkit Programming (I)                 *
 *            - Programming Basics, Resource and Event -               *
 *                                                                     *
 *      Copyright 1994, Hideyuki Hayashi                               *
 *                                                                     *
 *      This program can be distributed without fee, provided          *
 *      that the above copyright notice appear in all copies.          *
 ***********************************************************************/

#include <Xm/XmAll.h>

#define FRAGMENT    2    /* Amount of one task (%) */
static XtAppContext app_context;
static Widget dialog;
static Boolean cancelled = False;  /* The task is cancelled */
static int progress;     /* Progress:  0 = Start,  100 = Finished */

/* Show the progress (%) on the label */
static void ShowProgress(w, percent)
    Widget w;        /* Widget ID of the label */
    int    percent;  /* % of progress */
{
    char     string[BUFSIZ];
    XmString labelstr;

    sprintf(string, "%d %%", percent);  /* Create the string for % */
    labelstr = XmStringCreateLocalized(string); /* Compound string */
    XtVaSetValues(w, XmNmessageString, labelstr, NULL);  /* show on the label */
    XmStringFree(labelstr);  /* It's not necessary */
}

/* Called when "Start" is pressed */
static void StartCB(w, client_data, call_data)
    Widget    w;
    XtPointer client_data;
    XtPointer call_data;
{
    int    i, j;
    XEvent event;

    progress = 0;  /* Reset the progress */
    cancelled = False;  /* not cancelled */
    ShowProgress(dialog, progress);  /* show as "0 %" */
    XtManageChild(dialog);  /* show working dialog */

    /* repeat until the task is finished or cancelled */
    while (progress < 100 && cancelled != True) {
        while (XtAppPending(app_context)) { /* repeat until no event is qued */
            XtAppNextEvent(app_context, &event);  /* retrieve one event */
            XtDispatchEvent(&event);      /* process the event */
        }

        for (i = 0; i < 5000000; i++)  /* perform one fragment of the work */
            j = i * i;  /* Replace this line with fragment of your actual work*/

        progress += FRAGMENT;  /* update the progress percentage */
        ShowProgress(dialog, progress);  /* show the progress on the label */
    }

    XtUnmanageChild(dialog);  /* close the working dialog */
}

/* Called when "Cancel" button of the dialog is pressed */
static void CancelCB(w, client_data, call_data)
    Widget    w;
    XtPointer client_data;
    XtPointer call_data;
{
    cancelled = True;  /* The task is cancelled */
}

main(argc, argv)
    int  argc;
    char **argv;
{
    Widget toplevel, start;
    Arg    al[20];
    int    ac;

    XtSetLanguageProc(NULL, NULL, NULL);
    ac = 0;
    toplevel = XtAppInitialize(&app_context, "Workproc", NULL, 0,
                               &argc, argv, NULL, al, ac);
    /* Create working dialog */
    ac = 0;                 /* set the dialog mode to full application modal */
    XtSetArg(al[ac], XmNdialogStyle, XmDIALOG_FULL_APPLICATION_MODAL); ac++;
    dialog = XmCreateWorkingDialog(toplevel, "dialog", al, ac);
    /* "OK" and "Help" button is not necessary */
    XtUnmanageChild(XmMessageBoxGetChild(dialog, XmDIALOG_OK_BUTTON));
    XtUnmanageChild(XmMessageBoxGetChild(dialog, XmDIALOG_HELP_BUTTON));
    /* Register callback for "Cancel" button */
    XtAddCallback(dialog, XmNcancelCallback, CancelCB, NULL);

    ac = 0;
    start = XmCreatePushButton(toplevel, "start", al, ac);
    XtManageChild(start);
    XtAddCallback(start, XmNactivateCallback, StartCB, NULL);

    XtRealizeWidget(toplevel);
    XtAppMainLoop(app_context);
}
